unit frmmain;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, ComCtrls, Grids, libtar;

type
  { TMainForm }
  PTarDirRec = ^TTarDirRec;

  TMainForm = class(TForm)
    AClose: TAction;
    AExtract: TAction;
    AOPen: TAction;
    AQuit: TAction;
    ALTar: TActionList;
    MenuItem1: TMenuItem;
    MIExtract: TMenuItem;
    MIClose: TMenuItem;
    MISep1: TMenuItem;
    MIQuit: TMenuItem;
    MIOpen: TMenuItem;
    MMTar: TMainMenu;
    ODTar: TOpenDialog;
    SDExtract: TSelectDirectoryDialog;
    LVTAR : TListView;
    procedure AExtractExecute(Sender: TObject);
    procedure AExtractUpdate(Sender: TObject);
    procedure AOPenExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FTar : TTarArchive;
    procedure ClearData;
    procedure ExtractFile(Const ATargetDir : String; Const D : TTarDirRec);
    procedure ExtractToDir(ATargetDir: String);
    procedure OpenArchive(const AFileName: String);
    procedure ShowEntries;
    procedure ShowEntry(LI: TListItem; Const D : TTarDirRec);
    { private declarations }
  public
    { public declarations }
  end; 

var
  MainForm: TMainForm;

implementation

{ TMainForm }
resourcestring
  SViewingFile = 'Viewing tar file: %s';
  SErrCreatingDir = 'Could not create directory "%s"';

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  ClearData;
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  LVTar.ViewStyle:=vsReport;
end;

procedure TMainForm.AOPenExecute(Sender: TObject);
begin
  If ODTar.Execute then
    OpenArchive(ODTar.FileName);
end;

procedure TMainForm.AExtractUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=(FTar<>Nil) and (LVTar.SelCount>0);
end;

procedure TMainForm.AExtractExecute(Sender: TObject);
begin
  With SDExtract do
    if Execute then
      ExtractToDir(IncludeTrailingPathDelimiter(FileName));
end;

procedure TMainForm.ExtractToDir(ATargetDir : String);

Var
  L : TStrings;
  I : Integer;
  PD : PTarDirRec;
  D : TTarDirRec;

begin
  L:=TStringList.Create;
  try
    For I:=0 to LVTar.Items.Count-1 do
      begin
      If LVTar.Items[i].Selected then
        begin
        PD:=PTarDirRec(LVTar.Items[i].Data);
        L.Add(PD^.Name);
        end;
      end;
    FTar.Reset;
    While (L.Count>0) and FTar.FindNext(D) do
      begin
      I:=L.IndexOf(D.Name);
      If (I<>-1) then
        begin
        ExtractFile(ATargetDir,D);
        L.Delete(I);
        end;
      end;
  finally
    L.Free;
  end;
end;

procedure TMainForm.ExtractFile(Const ATargetDir : String; Const D : TTarDirRec);

Var
  ADir : String;
  AFileName : String;

begin
  AFileName:=ATargetDir+D.Name;
  if (D.FileType=ftNormal) then
    ADir:=ExtractFilePath(AFileName)
  else if (D.FileType=ftDirectory) then
    ADir:=AFileName;
  If Not ForceDirectories(ADir) then
    Raise Exception.CreateFmt(SErrCreatingDir,[ADir]);
  If (D.FileType=FTNormal) then
    FTar.ReadFile(AFileName);
end;


procedure TMainForm.ClearData;

Var
  I : Integer;

begin
  For I:=0 to LVTar.Items.Count-1 do
    FreeMem(LVTar.Items[i].Data);
  LVTar.Clear;
  FreeAndNil(FTar);
end;

procedure TMainForm.OpenArchive(Const AFileName : String);

begin
  ClearData;
  FTar:=TTarArchive.Create(AFileName);
  ShowEntries;
  Caption:=Format(SViewingFile,[AFileName]);
end;

procedure TMainForm.ShowEntries;

Var
  I : Integer;
  LI : TListItem;
  D : TTarDirRec;

begin
  LVTar.Items.BeginUpdate;
  try
    While FTar.FindNext(D) do
      begin
      LI:=LVTar.Items.Add;
      ShowEntry(LI,D);
      end;
  finally
    LVTar.Items.EndUpdate;
  end;
end;

procedure TMainForm.ShowEntry(LI : TListItem; Const D : TTarDirRec);

Var
  S : String;
  E : PTarDirRec;

begin
  S:=ExtractFileName(D.Name);
  If (S='') then // Directory entry
    S:=D.Name;
  LI.Caption:=S;
  With LI.SubItems do
    begin
    BeginUpdate;
    try
      Add(ExtractFileExt(D.Name));
      Add(IntToStr(D.Size));
      Add(DateTimeToStr(D.DateTime));
      Add(ExtractFilePath(D.Name));
    finally
      EndUpdate;
    end;
    end;
  New(E);
  E^:=D;
  LI.Data:=E;
end;



initialization
  {$I frmmain.lrs}

end.

