unit frmmaingrid;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, LResources, Forms, Controls, Graphics, Dialogs,
  Menus, ActnList, ComCtrls, Grids, libtar;

type
  { TMainForm }
  PTarDirRec = ^TTarDirRec;

  TMainForm = class(TForm)
    AClose: TAction;
    AExtract: TAction;
    AOPen: TAction;
    AQuit: TAction;
    ALTar: TActionList;
    MenuItem1: TMenuItem;
    MIExtract: TMenuItem;
    MIClose: TMenuItem;
    MISep1: TMenuItem;
    MIQuit: TMenuItem;
    MIOpen: TMenuItem;
    MMTar: TMainMenu;
    ODTar: TOpenDialog;
    SDExtract: TSelectDirectoryDialog;
    SGTar: TStringGrid;
    procedure AExtractExecute(Sender: TObject);
    procedure AExtractUpdate(Sender: TObject);
    procedure AOPenExecute(Sender: TObject);
    procedure AQuitExecute(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FTar : TTarArchive;
    FEntries : TFPList;
    procedure ClearData;
    procedure ExtractFile(Const ATargetDir : String; Const D : TTarDirRec);
    procedure ExtractToDir(ATargetDir: String);
    procedure OpenArchive(const AFileName: String);
    procedure ShowEntries;
    procedure ShowEntry(ARow : Integer; Const D : TTarDirRec);
    { private declarations }
  public
    { public declarations }
  end; 

var
  MainForm: TMainForm;

implementation

{ TMainForm }
resourcestring
  SViewingFile = 'Viewing tar file: %s';
  SErrCreatingDir = 'Could not create directory "%s"';

procedure TMainForm.AQuitExecute(Sender: TObject);
begin
  ClearData;
  Close;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  FEntries:=TFPList.Create;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  CLearData;
  FreeAndNil(FEntries);
end;

procedure TMainForm.AOPenExecute(Sender: TObject);
begin
  If ODTar.Execute then
    OpenArchive(ODTar.FileName);
end;

procedure TMainForm.AExtractUpdate(Sender: TObject);
begin
  (Sender as TAction).Enabled:=(FTar<>Nil) and (SGTar.Row>0);
end;

procedure TMainForm.AExtractExecute(Sender: TObject);
begin
  With SDExtract do
    if Execute then
      ExtractToDir(IncludeTrailingPathDelimiter(FileName));
end;

procedure TMainForm.ExtractToDir(ATargetDir : String);

Var
  L : TStrings;
  I : Integer;
  PD : PTarDirRec;
  D : TTarDirRec;

begin
  L:=TStringList.Create;
  try
    For I:=SGTar.Selection.Top to SGTar.Selection.Bottom do
      begin
      PD:=PTarDirRec(FEntries[i-1]);
      L.Add(PD^.Name);
      end;
    FTar.Reset;
    While (L.Count>0) and FTar.FindNext(D) do
      begin
      I:=L.IndexOf(D.Name);
      If (I<>-1) then
        begin
        ExtractFile(ATargetDir,D);
        L.Delete(I);
        end;
      end;
  finally
    L.Free;
  end;
end;

procedure TMainForm.ExtractFile(Const ATargetDir : String; Const D : TTarDirRec);

Var
  ADir : String;
  AFileName : String;

begin
  AFileName:=ATargetDir+D.Name;
  if (D.FileType=ftNormal) then
    ADir:=ExtractFilePath(AFileName)
  else if (D.FileType=ftDirectory) then
    ADir:=AFileName;
  If Not ForceDirectories(ADir) then
    Raise Exception.CreateFmt(SErrCreatingDir,[ADir]);
  If (D.FileType=FTNormal) then
    FTar.ReadFile(AFileName);
end;


procedure TMainForm.ClearData;

Var
  I : Integer;

begin
  For I:=0 to FEntries.Count-1 do
    FreeMem(FEntries[i]);
  FEntries.Clear;
  SGTar.RowCount:=2;
  SGTar.Rows[1].Clear;
  FreeAndNil(FTar);
end;

procedure TMainForm.OpenArchive(Const AFileName : String);

begin
  ClearData;
  FTar:=TTarArchive.Create(AFileName);
  ShowEntries;
  Caption:=Format(SViewingFile,[AFileName]);
end;

procedure TMainForm.ShowEntries;

Var
  I : Integer;
  LI : TListItem;
  D : TTarDirRec;
  E : PTarDirRec;

begin
  While FTar.FindNext(D) do
    begin
    New(E);
    E^:=D;
    FEntries.Add(E);
    end;
  SGTar.RowCount:=FEntries.Count+1;
  For I:=0 to FEntries.Count-1 do
    ShowEntry(I+1,PTarDirRec(FEntries[i])^);
end;

procedure TMainForm.ShowEntry(ARow : Integer; Const D : TTarDirRec);

Var
  S : String;
  L : TStrings;

begin
  S:=ExtractFileName(D.Name);
  If (S='') then // Directory entry
    S:=D.Name;
  L:=TStringList.Create;
  try
    With L do
      begin
      Add(S);
      Add(ExtractFileExt(D.Name));
      Add(IntToStr(D.Size));
      Add(DateTimeToStr(D.DateTime));
      Add(ExtractFilePath(D.Name));
      end;
    SGTar.Rows[ARow]:=L;
  finally
    L.Free;
  end;
end;



initialization
  {$I frmmaingrid.lrs}

end.
Pa
